<?php

namespace Drupal\dppca;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\dppca\Plugin\DPPCA\Unavailable;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for DPPCA task plugins providing generic functionality.
 */
abstract class DppcaTaskBasePlugin extends PluginBase implements DppcaTaskInterface, ContainerFactoryPluginInterface {

  /**
   * The module's config.
   *
   * @var \Drupal\dppca\Config
   */
  protected Config $config;

  /**
   * The module's storage.
   *
   * @var \Drupal\dppca\Storage
   */
  protected Storage $storage;

  /**
   * The plugin manager for DPPCA tasks.
   *
   * @var \Drupal\dppca\DppcaTaskPluginManager
   */
  protected DppcaTaskPluginManager $pluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    // @phpstan-ignore-next-line
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->config = $container->get('dppca.config');
    $instance->storage = $container->get('dppca.storage');
    $instance->pluginManager = $container->get('plugin.manager.dppca');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function label(): TranslatableMarkup {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function description(): TranslatableMarkup {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function category(): TranslatableMarkup {
    return $this->pluginDefinition['category'];
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $children): array {
    if ($children) {
      $build = [
        '#type' => 'details',
        '#title' => $this->label(),
        '#open' => FALSE,
        '#weight' => -1,
      ];
      foreach ($children as $id => $subChildren) {
        /** @var \Drupal\dppca\DppcaTaskInterface $plugin */
        $plugin = $this->pluginManager->createInstance($id);
        if ($plugin instanceof Unavailable) {
          $build[] = $this->buildMissingPlugin($id);
        }
        else {
          $build[] = $plugin->build($subChildren);
        }
      }
    }
    else {
      $build = [
        '#type' => 'fieldset',
        '#title' => $this->label(),
        '#weight' => 1,
        0 => $this->buildReport(),
      ];
    }
    return $build;
  }

  /**
   * Build the render array for the scan result of this plugin.
   *
   * @return array
   *   The render array.
   */
  protected function buildReport(): array {
    $status = $this->storage->getStatus($this->pluginId);
    return [
      '#markup' => 'Here will be the detailed report: ' . $status,
    ];
  }

  /**
   * Build the render array for a plugin that isn't available.
   *
   * @param string $id
   *   The plugin ID.
   *
   * @return array
   *   The render array.
   */
  protected function buildMissingPlugin(string $id): array {
    $plugin = $this->pluginManager->getPluginDefinition($id);
    if (empty($plugin['modules'])) {
      $suggestions = $this->t('Unfortunately, there is no suggested module known.');
    }
    else {
      $suggestions = $this->t('You may install one of the following modules:&nbsp;') . implode(', ', $plugin['modules']);
    }

    return [
      '#type' => 'fieldset',
      '#title' => $plugin['label'],
      '#weight' => 1,
      0 => [
        '#markup' => 'This plugin is missing. ' . $suggestions,
        '#prefix' => '<p>',
        '#suffix' => '</p>',
        '#allowed_tags' => ['a', 'em', 'small'],
      ],
    ];
  }

}
