<?php

namespace Drupal\dppca;

use Drupal\Component\Plugin\CategorizingPluginManagerInterface;
use Drupal\Component\Plugin\FallbackPluginManagerInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\CategorizingPluginManagerTrait;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\dppca\Attribute\DppcaTask;

/**
 * Provides a plugin manager for DPPCA tasks.
 */
class DppcaTaskPluginManager extends DefaultPluginManager implements FallbackPluginManagerInterface, CategorizingPluginManagerInterface {

  use CategorizingPluginManagerTrait;

  /**
   * DPPCA plugin manager constructor.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/DPPCA', $namespaces, $module_handler, DppcaTaskInterface::class, DppcaTask::class);
    $this->setCacheBackend($cache_backend, 'dppca_task');
    $this->alterInfo('dppca_task');
  }

  /**
   * {@inheritdoc}
   */
  public function getFallbackPluginId($plugin_id, array $configuration = []): string {
    return 'unavailable';
  }

  /**
   * Build the flat list of all existing and referenced plugins.
   *
   * @return array
   *   The list of all plugins.
   */
  protected function buildFullList(): array {
    $plugins = [];
    foreach ($this->getDefinitions() as $id => $definition) {
      if ($id === 'unavailable') {
        continue;
      }
      $plugins[$id] = $definition;
      $plugins[$id]['required_by'] = [];
    }
    foreach ($plugins as $definition) {
      foreach ($definition['require'] ?? [] as $requiredPlugin) {
        if (!isset($plugins[$requiredPlugin->id])) {
          $plugins[$requiredPlugin->id] = [
            'id' => $requiredPlugin->id,
            'label' => $requiredPlugin->label ?? $requiredPlugin->id,
            'modules' => $requiredPlugin->modules ?? [],
            'required_by' => [],
          ];
        }
      }
    }
    foreach ($plugins as $id => $definition) {
      foreach ($definition['require'] ?? [] as $requiredPlugin) {
        $plugins[$requiredPlugin->id]['required_by'][] = $id;
      }
    }
    return $plugins;
  }

  /**
   * Generates the tree of plugins.
   *
   * @return array
   *   The tree with plugin IDs.
   */
  public function generateTree(): array {
    $tree = [];
    $plugins = $this->buildFullList();
    foreach ($plugins as $id => $definition) {
      if (!$definition['required_by']) {
        $tree[$id] = [];
        $this->appendToTree($plugins, $tree[$id], $id);
      }
    }
    return $tree;
  }

  /**
   * Get the definition of a plugin, including the referenced ones.
   *
   * @param string $id
   *   The plugin ID.
   *
   * @return array
   *   The plugin definition.
   */
  public function getPluginDefinition(string $id): array {
    return $this->buildFullList()[$id];
  }

  /**
   * Helper function to recursively append required plugins to the tree.
   *
   * @param array $plugins
   *   All plugins.
   * @param array $treeNode
   *   Current item in the tree.
   * @param string $id
   *   Current ID.
   */
  private function appendToTree(array $plugins, array &$treeNode, string $id): void {
    foreach ($plugins[$id]['require'] ?? [] as $requiredPlugin) {
      $treeNode[$requiredPlugin->id] = [];
      $this->appendToTree($plugins, $treeNode[$requiredPlugin->id], $requiredPlugin->id);
    }
  }

}
